% Run IV LASSO program for eminent domain example.  Aggregate data.

% Run LASSO for first stage reported in Table 1, Panel C, Columns 4-6

clear ;

c = 1.1; % LASSO penalty multiplier

%% Read in data.  non-metro - panel norm; circuit specific trends
names1 = importfile('../EminentDomain/data/temp_HomePrices_nonmetro.mat.csv');
% NOTE: importfile cannot handle text data
cases = 'HPI_nonmetro, Panel';

% Read in additional probability controls 
prnames = importfile('../EminentDomain/data/probs.txt');

% Match new probability controls up by circuit and year
for kk = 3:size(prnames,1),
    assignin('base',strcat('F_',prnames{kk}),zeros(size(circuit)));
end
for ii = 1:max(circuit),
    for jj = min(year):max(year),
        I = find(Scircuit == ii & Syear == jj); %#ok<NASGU>
        J = find(circuit == ii & year == jj);
        for kk = 3:size(prnames,1),
            eval(strcat('F_',prnames{kk},'(J) = ',prnames{kk},'(I);'));
        end
    end
end
        

% Recode year and circuit dummy variables and prevent rank deficiency
yearn = nominal(year);
iyearn = dummyvar(yearn);
circuitn = nominal(circuit);
icircuitn = dummyvar(circuitn);
icircuitn(:,1) = [];
iyearn(:,1) = [];
col1 = ones(size(iyearn,1),1);
jcircuityear = zeros(size(icircuitn));
for i = 1:size(icircuitn, 2),
    jcircuityear(:,i) = year.*icircuitn(:,i);
end

cy = nominal((recode(circuit)-1).*max(recode(year))+recode(year));
Dt = dummyvar(cy);

% Circuit controls (prob_x_[CHAR]_1plus
circctrl = []; %Omitting for now
% Endogenous, d
d = numpro_casecat_12;
% Exogenous to be partialed out, x
x = [missing_cy_12, numcasecat_12, circctrl, icircuitn, iyearn, jcircuityear, col1];
% xP = x;
% for kk = 3:size(prnames,1),
%     xP = [xP eval(strcat('F_',prnames{kk}))]; %#ok<AGROW>
% end
% x = xP;
% Candidate instruments, z:
z = 1*[numpanels1x_noreligion, numpanels1x_jd_public, ...
    numpanels1x_dem, numpanels2x_dem, numpanels3x_dem, ...
    numpanels1x_dem.^2 , numpanels1x_dem.^3 , ...
    numpanels1x_female, numpanels2x_female, numpanels3x_female, ...
    numpanels1x_nonwhite, numpanels2x_nonwhite, numpanels3x_nonwhite, ...
    numpanels1x_black, numpanels2x_black, numpanels3x_black, ...
    numpanels1x_jewish, numpanels2x_jewish, numpanels3x_jewish, ...
    numpanels1x_catholic, numpanels2x_catholic, numpanels3x_catholic, ...
    numpanels2x_noreligion, numpanels3x_noreligion, ...
    numpanels1x_instate_ba, numpanels2x_instate_ba, numpanels3x_instate_ba, ...
    numpanels1x_ba_public, numpanels2x_ba_public, numpanels3x_ba_public, ...
    numpanels2x_jd_public, numpanels3x_jd_public, ...
    (numpanels1x_jd_public.^2)/5 , (numpanels1x_jd_public.^3)/25 , ...
    numpanels1x_elev, numpanels2x_elev, numpanels3x_elev, ...
    (numpanels1x_elev.^2) , (numpanels1x_elev.^3) , ...
    numpanels1x_female_black, numpanels2x_female_black, numpanels3x_female_black, ...
    numpanels1x_female_noreligion, numpanels2x_female_noreligion, numpanels3x_female_noreligion, ...
    numpanels1x_black_noreligion, numpanels2x_black_noreligion, numpanels3x_black_noreligion, ...
    numpanels1x_female_jd_public, numpanels2x_female_jd_public, numpanels3x_female_jd_public, ...
    numpanels1x_black_jd_public, numpanels2x_black_jd_public, numpanels3x_black_jd_public, ...
    numpanels1x_noreligion_jd_pub, numpanels2x_noreligion_jd_pub, numpanels3x_noreligion_jd_pub, ...
    numpanels1x_mainline, numpanels2x_mainline, numpanels3x_mainline, ...
    numpanels1x_evangelical, numpanels2x_evangelical, numpanels3x_evangelical, ...
    numpanels1x_protestant, numpanels2x_protestant, numpanels3x_protestant, ...
    numpanels1x_dem.*numpanels1x_female , numpanels1x_dem.*numpanels1x_black , ...
    numpanels1x_dem.*numpanels1x_jewish , numpanels1x_dem.*numpanels1x_catholic , ...
    numpanels1x_dem.*numpanels1x_noreligion , numpanels1x_dem.*numpanels1x_instate_ba , ...
    numpanels1x_dem.*numpanels1x_jd_public , numpanels1x_dem.*numpanels1x_mainline , ...
    numpanels1x_dem.*numpanels1x_evangelical , numpanels1x_dem.*numpanels1x_protestant , ...
    numpanels1x_dem.*numpanels1x_ba_public , numpanels1x_dem.*numpanels1x_elev , ...
    numpanels1x_dem.*numpanels1x_nonwhite , ...
    numpanels1x_female.*numpanels1x_jewish , numpanels1x_female.*numpanels1x_catholic , ...
    numpanels1x_female.*numpanels1x_noreligion , numpanels1x_female.*numpanels1x_instate_ba , ...
    numpanels1x_female.*numpanels1x_jd_public , numpanels1x_female.*numpanels1x_mainline , ...
    numpanels1x_female.*numpanels1x_evangelical , numpanels1x_female.*numpanels1x_protestant , ...
    numpanels1x_female.*numpanels1x_ba_public , numpanels1x_female.*numpanels1x_elev , ...
    numpanels1x_female.*numpanels1x_nonwhite , ...
    numpanels1x_jd_public.*numpanels1x_jewish , numpanels1x_jd_public.*numpanels1x_catholic , ...
    numpanels1x_jd_public.*numpanels1x_noreligion , numpanels1x_jd_public.*numpanels1x_instate_ba , ...
    numpanels1x_jd_public.*numpanels1x_mainline , ...
    numpanels1x_jd_public.*numpanels1x_evangelical , numpanels1x_jd_public.*numpanels1x_protestant , ...
    numpanels1x_jd_public.*numpanels1x_ba_public , numpanels1x_jd_public.*numpanels1x_elev , ...
    numpanels1x_jd_public.*numpanels1x_nonwhite , ...
    numpanels1x_ba_public.*numpanels1x_jewish , numpanels1x_ba_public.*numpanels1x_catholic , ...
    numpanels1x_ba_public.*numpanels1x_noreligion , numpanels1x_ba_public.*numpanels1x_instate_ba , ...
    numpanels1x_ba_public.*numpanels1x_mainline , ...
    numpanels1x_ba_public.*numpanels1x_evangelical , numpanels1x_ba_public.*numpanels1x_protestant , ...
    numpanels1x_ba_public.*numpanels1x_elev , ...
    numpanels1x_ba_public.*numpanels1x_nonwhite , ...
    numpanels2x_jd_public.*numpanels1x_jewish , numpanels2x_jd_public.*numpanels1x_catholic , ...
    numpanels2x_jd_public.*numpanels1x_noreligion , numpanels2x_jd_public.*numpanels1x_instate_ba , ...
    numpanels2x_jd_public.*numpanels1x_mainline , ...
    numpanels2x_jd_public.*numpanels1x_evangelical , numpanels2x_jd_public.*numpanels1x_protestant , ...
    numpanels2x_jd_public.*numpanels1x_ba_public , numpanels2x_jd_public.*numpanels1x_elev , ...
    numpanels2x_jd_public.*numpanels1x_nonwhite , ...
    numpanels3x_jd_public.*numpanels1x_jewish , numpanels3x_jd_public.*numpanels1x_catholic , ...
    numpanels3x_jd_public.*numpanels1x_noreligion , numpanels3x_jd_public.*numpanels1x_instate_ba , ...
    numpanels3x_jd_public.*numpanels1x_mainline , ...
    numpanels3x_jd_public.*numpanels1x_evangelical , numpanels3x_jd_public.*numpanels1x_protestant , ...
    numpanels3x_jd_public.*numpanels1x_ba_public , numpanels3x_jd_public.*numpanels1x_elev , ...
    numpanels3x_jd_public.*numpanels1x_nonwhite , ...
    numpanels2x_female.*numpanels1x_jewish , numpanels2x_female.*numpanels1x_catholic , ...
    numpanels2x_female.*numpanels1x_noreligion , numpanels2x_female.*numpanels1x_instate_ba , ...
    numpanels2x_female.*numpanels1x_jd_public , numpanels2x_female.*numpanels1x_mainline , ...
    numpanels2x_female.*numpanels1x_evangelical , numpanels2x_female.*numpanels1x_protestant , ...
    numpanels2x_female.*numpanels1x_ba_public , numpanels2x_female.*numpanels1x_elev , ...
    numpanels2x_female.*numpanels1x_nonwhite , ...
    numpanels2x_ba_public.*numpanels1x_jewish , numpanels2x_ba_public.*numpanels1x_catholic , ...
    numpanels2x_ba_public.*numpanels1x_noreligion , numpanels2x_ba_public.*numpanels1x_instate_ba , ...
    numpanels2x_ba_public.*numpanels1x_mainline , ...
    numpanels2x_ba_public.*numpanels1x_evangelical , numpanels2x_ba_public.*numpanels1x_protestant , ...
    numpanels2x_ba_public.*numpanels1x_elev , ...
    numpanels2x_ba_public.*numpanels1x_nonwhite , ...
    numpanels1x_instate_ba.*numpanels1x_jewish , numpanels1x_instate_ba.*numpanels1x_catholic , ...
    numpanels1x_instate_ba.*numpanels1x_noreligion , numpanels1x_instate_ba.*numpanels1x_instate_ba , ...
    numpanels1x_instate_ba.*numpanels1x_mainline , ...
    numpanels1x_instate_ba.*numpanels1x_evangelical , numpanels1x_instate_ba.*numpanels1x_protestant , ...
    numpanels1x_instate_ba.*numpanels1x_elev , ...
    numpanels1x_instate_ba.*numpanels1x_nonwhite , ...
    numpanels1x_elev.*numpanels1x_jewish , numpanels1x_elev.*numpanels1x_catholic , ...
    numpanels1x_elev.*numpanels1x_noreligion , ...
    numpanels1x_elev.*numpanels1x_mainline , ...
    numpanels1x_elev.*numpanels1x_evangelical , numpanels1x_elev.*numpanels1x_protestant , ...
    numpanels1x_elev.*numpanels1x_nonwhite , ...
    numpanels1x_protestant.*numpanels1x_jewish , numpanels1x_protestant.*numpanels1x_catholic , ...
    numpanels1x_protestant.*numpanels1x_noreligion , ...
    numpanels1x_protestant.*numpanels1x_mainline , ...
    numpanels1x_protestant.*numpanels1x_evangelical , numpanels1x_protestant.*numpanels1x_protestant , ...
    numpanels1x_protestant.*numpanels1x_nonwhite , ...
    numpanels1x_mainline.*numpanels1x_jewish , numpanels1x_mainline.*numpanels1x_catholic , ...
    numpanels1x_mainline.*numpanels1x_noreligion , numpanels1x_mainline.*numpanels1x_evangelical , ...
    numpanels1x_mainline.*numpanels1x_mainline , ...
    numpanels1x_mainline.*numpanels1x_nonwhite ];
% Candidate instruments' names, namez
namez = {'1x_noreligion', '1x_jd_public', '1x_dem', '2x_dem', '3x_dem', ...
    '1x_dem^2', '1x_dem^3', ...
    '1x_female', '2x_female', '3x_female', ...
    '1x_nonwhite', '2x_nonwhite', '3x_nonwhite', ...
    '1x_black', '2x_black', '3x_black', ...
    '1x_jewish', '2x_jewish', '3x_jewish', ...
    '1x_catholic', '2x_catholic', '3x_catholic', ...
    '2x_noreligion', '3x_noreligion', ...
    '1x_instate_ba', '2x_instate_ba', '3x_instate_ba', ...
    '1x_ba_public', '2x_ba_public', '3x_ba_public', ...
    '2x_jd_public', '3x_jd_public', ...
    '1x_jd_public^2', '1x_jd_public^3' , ...
    '1x_elev', '2x_elev', '3x_elev', ...
    '1x_elev^2' , '1x_elev^3' , ...
    '1x_female_black', '2x_female_black', '3x_female_black', ...
    '1x_female_noreligion', '2x_female_noreligion', '3x_female_noreligion', ...
    '1x_black_noreligion', '2x_black_noreligion', '3x_black_noreligion', ...
    '1x_female_jd_public', '2x_female_jd_public', '3x_female_jd_public', ...
    '1x_black_jd_public', '2x_black_jd_public', '3x_black_jd_public', ...
    '1x_noreligion_jd_pub', '2x_noreligion_jd_pub', '3x_noreligion_jd_pub', ...
    '1x_mainline', '2x_mainline', '3x_mainline', ...
    '1x_evangelical', '2x_evangelical', '3x_evangelical', ...
    '1x_protestant', '2x_protestant', '3x_protestant', ...
    '1x_dem.*1x_female' , '1x_dem.*1x_black' , ...
    '1x_dem.*1x_jewish' , '1x_dem.*1x_catholic' , ...
    '1x_dem.*1x_noreligion' , '1x_dem.*1x_ba_public' , ...
    '1x_dem.*1x_jd_public' , '1x_dem.*1x_mainline' , ...
    '1x_dem.*1x_evangelical' , '1x_dem.*1x_protestant' , ...
    '1x_dem.*1x_ba_public', '1x_dem.*1x_elev' , '1x_dem.*1x_nonwhite' , ... 
    '1x_female.*1x_jewish' , '1x_female.*1x_catholic' , ...
    '1x_female.*1x_noreligion' , '1x_female.*1x_ba_public' , ...
    '1x_female.*1x_jd_public' , '1x_female.*1x_mainline' , ...
    '1x_female.*1x_evangelical' , '1x_female.*1x_protestant' , ...
    '1x_female.*1x_ba_public', '1x_female.*1x_elev' , '1x_female.*1x_nonwhite' , ... 
    '1x_jd_public.*1x_jewish' , '1x_jd_public.*1x_catholic' , ...
    '1x_jd_public.*1x_noreligion' , '1x_jd_public.*1x_instate_ba' , ...
    '1x_jd_public.*1x_mainline' , ...
    '1x_jd_public.*1x_evangelical' , '1x_jd_public.*1x_protestant' , ...
    '1x_jd_public.*1x_ba_public', '1x_jd_public.*1x_elev' , ...
    '1x_jd_public.*1x_nonwhite' , ...
    '1x_ba_public.*1x_jewish' , '1x_ba_public.*1x_catholic' , ...
    '1x_ba_public.*1x_noreligion' , '1x_ba_public.*1x_instate_ba', ...
    '1x_ba_public.*1x_mainline' , ...
    '1x_ba_public.*1x_evangelical' , '1x_ba_public.*1x_protestant' , ...
    '1x_ba_public.*1x_elev' , ...
    '1x_ba_public.*1x_nonwhite' , ...
    '2x_jd_public.*1x_jewish' , '2x_jd_public.*1x_catholic' , ...
    '2x_jd_public.*1x_noreligion' , '2x_jd_public.*1x_instate_ba' , ...
    '2x_jd_public.*1x_mainline' , ...
    '2x_jd_public.*1x_evangelical' , '2x_jd_public.*1x_protestant' , ...
    '2x_jd_public.*1x_ba_public', '2x_jd_public.*1x_elev' , ...
    '2x_jd_public.*1x_nonwhite' , ...
    '3x_jd_public.*1x_jewish' , '3x_jd_public.*1x_catholic' , ...
    '3x_jd_public.*1x_noreligion' , '3x_jd_public.*1x_instate_ba' , ...
    '3x_jd_public.*1x_mainline' , ...
    '3x_jd_public.*1x_evangelical' , '3x_jd_public.*1x_protestant' , ...
    '3x_jd_public.*1x_ba_public', '3x_jd_public.*1x_elev' , ...
    '3x_jd_public.*1x_nonwhite' , ...
    '2x_female.*1x_jewish' , '2x_female.*1x_catholic' , ...
    '2x_female.*1x_noreligion' , '2x_female.*1x_ba_public' , ...
    '2x_female.*1x_jd_public' , '2x_female.*1x_mainline' , ...
    '2x_female.*1x_evangelical' , '2x_female.*1x_protestant' , ...
    '2x_female.*1x_ba_public', '2x_female.*1x_elev' , '2x_female.*1x_nonwhite' , ... 
    '2x_ba_public.*1x_jewish' , '2x_ba_public.*1x_catholic' , ...
    '2x_ba_public.*1x_noreligion' , '2x_ba_public.*1x_instate_ba', ...
    '2x_ba_public.*1x_mainline' , ...
    '2x_ba_public.*1x_evangelical' , '2x_ba_public.*1x_protestant' , ...
    '2x_ba_public.*1x_elev' , ...
    '2x_ba_public.*1x_nonwhite' , ...
    '1x_instate_ba.*1x_jewish' , '1x_instate_ba.*1x_catholic' , ...
    '1x_instate_ba.*1x_noreligion' , '1x_instate_ba.*1x_instate_ba', ...
    '1x_instate_ba.*1x_mainline' , ...
    '1x_instate_ba.*1x_evangelical' , '1x_instate_ba.*1x_protestant' , ...
    '1x_instate_ba.*1x_elev' , ...
    '1x_instate_ba.*1x_nonwhite' , ...    
    '1x_elev.*1x_jewish' , '1x_elev.*1x_catholic' , ...
    '1x_elev.*1x_noreligion' , '1x_elev.*1x_mainline' , ...
    '1x_elev.*1x_evangelical' , '1x_elev.*1x_protestant' , ...
    '1x_elev.*1x_nonwhite' , ...
    '1x_protestant.*1x_jewish' , '1x_protestant.*1x_catholic' , ...
    '1x_protestant.*1x_noreligion' , '1x_protestant.*1x_mainline' , ...
    '1x_protestant.*1x_evangelical' , '1x_protestant.*1x_protestant' , ...
    '1x_protestant.*1x_nonwhite' , ...
    '1x_mainline.*1x_jewish' , '1x_mainline.*1x_catholic' , ...
    '1x_mainline.*1x_noreligion' , ...
    '1x_mainline.*1x_evangelical' , '1x_mainline.*1x_mainline' , ...
    '1x_mainline.*1x_nonwhite' }';

% Probability controls, xP:
xP = 1*[F_prob_1x_dem, F_prob_2x_dem, F_prob_3x_dem, ...
    F_prob_1x_female, F_prob_2x_female, F_prob_3x_female, ...
    F_prob_1x_nonwhite, F_prob_2x_nonwhite, F_prob_3x_nonwhite, ...
    F_prob_1x_black, F_prob_2x_black, F_prob_3x_black, ...
    F_prob_1x_jewish, F_prob_2x_jewish, F_prob_3x_jewish, ...
    F_prob_1x_catholic, F_prob_2x_catholic, F_prob_3x_catholic, ...
    F_prob_1x_noreligion, F_prob_2x_noreligion, F_prob_3x_noreligion, ...
    F_prob_1x_instate_ba, F_prob_2x_instate_ba, F_prob_3x_instate_ba, ...
    F_prob_1x_ba_public, F_prob_2x_ba_public, F_prob_3x_ba_public, ...
    F_prob_1x_jd_public, F_prob_2x_jd_public, F_prob_3x_jd_public, ...
    F_prob_1x_elev, F_prob_2x_elev, F_prob_3x_elev, ...
    F_prob_1x_female_black, F_prob_2x_female_black, F_prob_3x_female_black, ...
    F_prob_1x_female_noreligion, F_prob_2x_female_noreligion, F_prob_3x_female_noreligion, ...
    F_prob_1x_black_noreligion, F_prob_2x_black_noreligion, F_prob_3x_black_noreligion, ...
    F_prob_1x_female_jd_public, F_prob_2x_female_jd_public, F_prob_3x_female_jd_public, ...
    F_prob_1x_black_jd_public, F_prob_2x_black_jd_public, F_prob_3x_black_jd_public, ...
    F_prob_1x_noreligion_jd_public, F_prob_2x_noreligion_jd_public, F_prob_3x_noreligion_jd_public, ...
    F_prob_1x_mainline, F_prob_2x_mainline, F_prob_3x_mainline, ...
    F_prob_1x_evangelical, F_prob_2x_evangelical, F_prob_3x_evangelical, ...
    F_prob_1x_protestant, F_prob_2x_protestant, F_prob_3x_protestant ];

% Names of Daniel's instruments
IND_dan = zeros(size(z,2),1);
IND_dan([1,2]) = 1;
IND_dan = logical(IND_dan);


%% LASSO runs with full set of probability controls
% Partial out x's that need to show up in both stages
I = find(mean(z) >= .05);
z = z(:,I);
IND_dan = IND_dan(I);
namez = namez(I);
y = loghpi_nonmetro;
I = find(mean(xP) >= .05);
xP = xP(:,I); %#ok<FNDSB>
x = [x xP];

% Collapse to circuit-year level
x = inv(Dt'*Dt)*(Dt'*x);
y = inv(Dt'*Dt)*(Dt'*y);
d = inv(Dt'*Dt)*(Dt'*d);
z = inv(Dt'*Dt)*(Dt'*z);

% Too many instruments, Fuller undefined.

xxinv = inv(x'*x);
My = full(y - x*xxinv*(x'*y)); %#ok<*MINV>
Md = full(d - x*xxinv*(x'*d));
Mz = full(z - x*xxinv*(x'*z));

I = find(std(Mz) > 1e-6);
Mz = Mz(:,I);
IND_dan = IND_dan(I);
namez = namez(I);

n = size(Mz,1);

% % Standardize for LASSO selection
sd = std(Md);
Sz_app = (ones(size(Mz,1),1)*std(Mz));
Md = Md/sd;
Mz = Mz./Sz_app;

K = size(Mz,2);

% Use LASSO to select instruments for several values of tuning parameter
% For now, three options:
%   1.  Select 2 instruments
%   2.  Select instruments using data-dependent penalty "A"
%   3.  Select instruments using data-dependent penalty "L"


% Select 2 instruments
[L,IND] = findLambdaK(Mz,Md,2,4000);
sum(IND)
v = Md - Mz(:,IND)*inv(Mz(:,IND)'*Mz(:,IND))*(Mz(:,IND)'*Md);

% Select with initial penalty loadings
St = Mz.*(v*ones(1,size(Mz,2)));
Ups = sqrt(sum(St.^2)/n);
lambda = c*2*sqrt(2*n*(log(2*(size(Mz,2)))));
PIpA = LassoShooting(Mz./(ones(n,1)*Ups), Md , lambda);
INDpA = ( abs(PIpA) > 1.0e-4 );
sum(INDpA)
v = Md - Mz(:,INDpA)*inv(Mz(:,INDpA)'*Mz(:,INDpA))*(Mz(:,INDpA)'*Md);

% Select with refined penalty loadings
St = Mz.*(v*ones(1,size(Mz,2)));
UpsR = sqrt(sum(St.^2)/n);
PIpL = LassoShooting(Mz./(ones(n,1)*UpsR), Md , lambda);
INDpL = ( abs(PIpL) > 1.0e-4 );
sum(INDpL)

% % Write output
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', {'varname' 'select 2' 'Init' 'Refined'}, cases, 'A1');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', namez, cases, 'A2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', IND, cases, 'B2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', INDpA, cases, 'C2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', INDpL, cases, 'D2');

%% Obtain FS and 2SLS estimates
% Return instruments and endogenous variables to original units for
% interpretation
Md = Md*sd;
Mz = Mz.*Sz_app;

% Number of nonredundant partialed out x's
kx = size(x,2);

% OLS Estimates
OLS = inv(Md'*Md)*(Md'*My);
et = My-Md*OLS;
sOLS = sqrt((n-1)/(n-kx-1))*hetero_se(Md,et,inv(Md'*Md));

% Calculate first stage
zt = Mz(:,IND_dan);
FS_dan = inv(zt'*zt)*(zt'*Md);
ut = Md-zt*FS_dan; 
[sFS_dan,VFS_dan] = hetero_se(zt,ut,inv(zt'*zt));
sFS_dan = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*sFS_dan;
VFS_dan = (n-size(zt,2))\(n-size(zt,2)-kx)*VFS_dan;
FS_F_dan = (FS_dan'*inv(VFS_dan)*FS_dan);  % ~chi2(2)
FS_p_dan = 1-chi2cdf(FS_F_dan,2);

% Calculate 2sls estimate
TSLS_dan = inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)*(zt'*My));
et = My-Md*TSLS_dan;
sTSLS_dan = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
    inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));


% Calculate results using LASSO choosing 2 instruments
% Calculate first stage
zt = Mz(:,IND);
FS_2 = inv(zt'*zt)*(zt'*Md);
ut = Md-zt*FS_2;
[sFS_2,VFS_2] = hetero_se(zt,ut,inv(zt'*zt));
sFS_2 = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*sFS_2;
VFS_2 = (n-size(zt,2))\(n-size(zt,2)-kx)*VFS_2;
FS_F_2 = (FS_2'*inv(VFS_2)*FS_2);  % ~chi2(2)
FS_p_2 = 1-chi2cdf(FS_F_2,2);

% Calculate 2sls estimate
TSLS_2 = inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)*(zt'*My));
et = My-Md*TSLS_2;
sTSLS_2 = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
    inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));

% Calculate results using LASSO A penalty
% Calculate first stage
zt = Mz(:,INDpA);
qA = sum(INDpA);
FS_A = inv(zt'*zt)*(zt'*Md);
ut = Md-zt*FS_A;
[sFS_A,VFS_A] = hetero_se(zt,ut,inv(zt'*zt));
sFS_A = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*sFS_A;
VFS_A = (n-size(zt,2))\(n-size(zt,2)-kx)*VFS_A;
FS_F_A = (FS_A'*inv(VFS_A)*FS_A);  % ~chi2(qA)
FS_p_A = 1-chi2cdf(FS_F_A,qA);

% Calculate 2sls estimate
TSLS_A = inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)*(zt'*My));
et = My-Md*TSLS_A;
sTSLS_A = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
    inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));

% Calculate results using LASSO L penalty
% Calculate first stage
zt = Mz(:,INDpL);
qL = sum(INDpL);
FS_L = pinv(zt'*zt)*(zt'*Md);
ut = Md-zt*FS_L;
[sFS_L,VFS_L] = hetero_se(zt,ut,inv(zt'*zt));
sFS_L = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*sFS_L;
VFS_L = (n-size(zt,2))\(n-size(zt,2)-kx)*VFS_L;
FS_F_L = (FS_L'*inv(VFS_L)*FS_L);  % ~chi2(qL)
FS_p_L = 1-chi2cdf(FS_F_L,qL);

% Calculate 2sls estimate
TSLS_L = pinv((Md'*zt)*pinv(zt'*zt)*(zt'*Md))*((Md'*zt)*pinv(zt'*zt)*(zt'*My));
et = My-Md*TSLS_L;
sTSLS_L = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
    inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));

% cases = 'HPI_nonmetro, Panel, Ests';
% % Write output
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', {'Row' 'OLS' 'Original' 'Select 2' 'Init' 'Refined'}, cases, 'A1');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', {'bhat' 'se' 'FS_F' 'FS_p' 'q' 'n' 'Instruments'}', cases, 'A2'); 
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', [OLS;sOLS;NaN;NaN;NaN;n], cases, 'B2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', [TSLS_dan;sTSLS_dan;FS_F_dan;FS_p_dan;2;n], cases, 'C2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', [TSLS_2;sTSLS_2;FS_F_2;FS_p_2;2;n], cases, 'D2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', [TSLS_A;sTSLS_A;FS_F_A;FS_p_A;qA;n], cases, 'E2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', [TSLS_L;sTSLS_L;FS_F_L;FS_p_L;qL;n], cases, 'F2');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', NaN, cases, 'B8');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', namez(IND_dan), cases, 'C8');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', namez(IND), cases, 'D8');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', namez(INDpA), cases, 'E8');
% xlswrite('../EminentDomain/output/LASSO_Aggregate.xls', namez(INDpL), cases, 'F8');


zt_dan = Mz(:,IND_dan);
et_dan = My-Md*TSLS_dan;
score_dan = zt_dan.*(et_dan*ones(1,2));
score_L = zt.*(et*ones(1,size(zt,2)));
se_dif = sqrt(sTSLS_dan^2 + sTSLS_L^2 - ...
    2*inv((Md'*zt_dan)*inv(zt_dan'*zt_dan)*(zt_dan'*Md))...
    *((Md'*zt_dan)*inv(zt_dan'*zt_dan))*(score_dan'*score_L)...
    *(inv(zt'*zt)*(zt'*Md))*inv((Md'*zt)*inv(zt'*zt)*(zt'*Md)));
t_dif = (TSLS_dan-TSLS_L)/se_dif;

disp('non-metro');
disp(t_dif);

% Calculate results using union of Daniel's instruments and LASSO
% instruments
INDunion = max(IND_dan,INDpL);
zt = Mz(:,INDunion);
qU = sum(INDunion);
FS_U = pinv(zt'*zt)*(zt'*Md);
ut = Md-zt*FS_U;
[sFS_U,VFS_U] = hetero_se(zt,ut,inv(zt'*zt));
sFS_U = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*sFS_U;
VFS_U = (n-size(zt,2))\(n-size(zt,2)-kx)*VFS_U;
FS_F_U = (FS_U'*inv(VFS_U)*FS_U);  % ~chi2(qU)
FS_p_U = 1-chi2cdf(FS_F_U,qU);

% Calculate 2sls estimate
TSLS_U = pinv((Md'*zt)*pinv(zt'*zt)*(zt'*Md))*((Md'*zt)*pinv(zt'*zt)*(zt'*My));
et = My-Md*TSLS_U;
sTSLS_U = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
    inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));




% Sup-Score Test
aVec = (-.5:.001:.5)';
SupScore = zeros(size(aVec));
for jj = 1:size(aVec,1)
    aT = aVec(jj,1);
    eTemp = My-Md*aT;
    ScoreVec = eTemp'*Mz;
    ScoreStd = sqrt((eTemp.^2)'*(Mz.^2));
    ScaledScore = ScoreVec./(1.1*ScoreStd);
    SupScore(jj,1) = max(abs(ScaledScore));
end
plot(aVec,SupScore);
ind = SupScore <= norminv(1-.05/(2*size(Mz,2)));
SupScoreInt = [min(aVec(ind)) max(aVec(ind))] %#ok<NOPTS>

